<?php

namespace Drupal\layout_settings\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class PredefinedClassesForm.
 */
class PredefinedClassesForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'layout_settings.predefined_classes',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'predefined_classes';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('layout_settings.predefined_classes');
    $description = '<p>' . t('The possible values this setting can contain. Enter one value per line, in the format class|label.');
    $description .= '<br/>' . t('The key is the stored value. The label will be used in displayed values and edit forms.');
    $form['predefined_classes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Classes'),
      '#default_value' => $config->get('predefined_classes'),
      '#description' => $description,
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $this->config('layout_settings.predefined_classes')
      ->set('predefined_classes', $form_state->getValue('predefined_classes'))
      ->save();
  }

}
