<?php

namespace Drupal\layout_settings\Plugin\LayoutSettings;


use Drupal\Component\Utility\Xss;
use Drupal\Core\Url;

trait BackgroundCssTrait {

  public static function defaultSettings() {
    return [
      'image_style' => '',
      'css_settings' => [
        'image_selector' => 'body',
        'image_color' => '#FFFFFF',
        'image_x' => 'left',
        'image_y' => 'top',
        'image_attachment' => 'scroll',
        'image_repeat' => 'no-repeat',
        'image_background_size' => '',
        'image_background_size_ie8' => 0,
        'image_gradient' => '',
        'image_media_query' => 'all',
        'image_important' => 1,
        'image_z_index' => 'auto',
        'image_path_format' => 'absolute',
      ],
    ];
  }

  public function settingsForm(array $settings) {
    $element = [];

    $element['image_style'] = [
      '#title' => $this->t('Image style'),
      '#type' => 'select',
      '#default_value' => isset($settings['image_style']) ? $settings['image_style'] : NULL,
      '#empty_option' => $this->t('None (original image)'),
      '#options' => image_style_options(),
      '#description' => $this->t(
        'Select <a href="@href_image_style">the image style</a> to apply on images',
        [
          '@href_image_style' => Url::fromRoute('image.style_add')->toString(),
        ]
      ),
    ];

    // Fieldset for css settings.
    $element['css_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Default CSS Settings'),
      '#description' => $this->t(
        'Default CSS settings for outputting the background property.
                These settings will be concatenated to form a complete css statementthat uses the "background"
                property. For more information on the css background property see
                http://www.w3schools.com/css/css_background.asp"'
      ),
    ];
    // The selector for the background property.
    $element['css_settings']['image_selector'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Selector(s)'),
      '#description' => $this->t(
        'A valid CSS selector that will be used to apply the background image. One per line.
                      If the field is a multivalue field, the first line will be applied to the first value,
                      the second to the second value... and so on. Tokens are supported.'
      ),
      '#default_value' => isset($settings['css_settings']['image_selector']) ? $settings['css_settings']['image_selector'] : NULL,
    ];
    // The token help relevant to this entity type.
    if (isset($form['#entity_type'])) {
      $element['css_settings']['token_help'] = [
        '#theme' => 'token_tree_link',
        '#token_types' => [],
      ];
    }
    else {
      $element['css_settings']['token_help'] = [
        '#theme' => 'token_tree_link',
      ];
    }

    // The selector for the background property.
    $element['css_settings']['image_z_index'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Z Index'),
      '#description' => $this->t(
        'The z-index property specifies the stack order of an element. An element with greater stack order is
                      always in front of an element with a lower stack order. Note: z-index only works on positioned
                      elements (position:absolute, position:relative, or position:fixed)'
      ),
      '#default_value' => isset($settings['css_settings']['image_z_index']) ? $settings['css_settings']['image_z_index'] : NULL,
    ];

    // The selector for the background property.
    //    $element['css_settings']['image_color'] = [
    //      '#type' => 'textarea',
    //      '#title' => $this->t('Color'),
    //      '#description' => $this->t(
    //        'The background color formatted as any valid css color format (e.g. hex, rgb, text, hsl)
    //                      [<a href="@url">css property: background-color</a>]. One per line. If the field is a multivalue
    //                      field, the first line will be applied to the first value, the second to the second value...
    //                      and so on.',
    //        ['@url' => 'https://developer.mozilla.org/en-US/docs/Web/CSS/linear-gradient']
    //      ),
    //      '#default_value' => $settings['css_settings']['image_color'],
    //    ];

    $element['css_settings']['image_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Color'),
      '#description' => $this->t('The background color formatted as any valid css color format (e.g. hex, rgb, text, hsl) [<a href="http://www.w3schools.com/css/pr_background-color.asp">css property: background-color</a>]'),
      '#default_value' => isset($settings['css_settings']['image_color']) ? $settings['css_settings']['image_color'] : NULL,
    ];


    // The selector for the background property.
    $element['css_settings']['image_x'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Horizontal Alignment'),
      '#description' => $this->t(
        'The horizontal alignment of the background image formatted as any valid css alignment.
                      [<a href="http://www.w3schools.com/css/pr_background-position.asp">
                      css property: background-position
                      </a>]'
      ),
      '#default_value' => isset($settings['css_settings']['image_x']) ? $settings['css_settings']['image_x'] : NULL,
    ];
    // The selector for the background property.
    $element['css_settings']['image_y'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Vertical Alignment'),
      '#description' => $this->t(
        'The vertical alignment of the background image formatted as any valid css alignment.
                      [<a href="http://www.w3schools.com/css/pr_background-position.asp">
                      css property: background-position
                      </a>]'
      ),
      '#default_value' => isset($settings['css_settings']['image_y']) ? $settings['css_settings']['image_y'] : NULL,
    ];
    // The selector for the background property.
    $element['css_settings']['image_attachment'] = [
      '#type' => 'radios',
      '#title' => $this->t('Background Attachment'),
      '#description' => $this->t(
        'The attachment setting for the background image.
                      [<a href="http://www.w3schools.com/css/pr_background-attachment.asp">
                      css property: background-attachment
                      </a>]'
      ),
      '#options' => [
        FALSE => $this->t('Ignore'),
        'scroll' => 'Scroll',
        'fixed' => 'Fixed',
      ],
      '#default_value' => isset($settings['css_settings']['image_attachment']) ? $settings['css_settings']['image_attachment'] : NULL,
    ];
    // The background-repeat property.
    $element['css_settings']['image_repeat'] = [
      '#type' => 'radios',
      '#title' => $this->t('Background Repeat'),
      '#description' => $this->t(
        'Define the repeat settings for the background image.
                      [<a href="http://www.w3schools.com/css/pr_background-repeat.asp">
                      css property: background-repeat
                      </a>]'
      ),
      '#options' => [
        FALSE => $this->t('Ignore'),
        'no-repeat' => $this->t('No Repeat'),
        'repeat' => $this->t('Tiled (repeat)'),
        'repeat-x' => $this->t('Repeat Horizontally (repeat-x)'),
        'repeat-y' => $this->t('Repeat Vertically (repeat-y)'),
      ],
      '#default_value' => isset($settings['css_settings']['image_repeat']) ? $settings['css_settings']['image_repeat'] : NULL,
    ];
    // The background-size property.
    $element['css_settings']['image_background_size'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Background Size'),
      '#description' => $this->t(
        'The size of the background (NOTE: CSS3 only. Useful for responsive designs)
                      [<a href="http://www.w3schools.com/cssref/css3_pr_background-size.asp">
                      css property: background-size
                      </a>]'
      ),
      '#default_value' => isset($settings['css_settings']['image_background_size']) ? $settings['css_settings']['image_background_size'] : NULL,
    ];
    // background-size:cover suppor for IE8.
    $element['css_settings']['image_background_size_ie8'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add background-size:cover support for ie8'),
      '#description' => $this->t(
        'The background-size css property is only supported on browsers that support CSS3.
                      However, there is a workaround for IE using Internet Explorer\'s built-in filters
                      (http://msdn.microsoft.com/en-us/library/ms532969%28v=vs.85%29.aspx).
                      Check this box to add the filters to the css. Sometimes it works well, sometimes it doesn\'t.
                      Use at your own risk'
      ),
      '#default_value' => isset($settings['css_settings']['image_background_size_ie8']) ? $settings['css_settings']['image_background_size_ie8'] : NULL,
    ];
    // Add gradient to background-image.
    $element['css_settings']['image_gradient'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Gradient'),
      '#description' => $this->t(
        'Apply this background image gradient css.
                  Example: linear-gradient(red, yellow)
                  [<a href="https://www.w3schools.com/css/css3_gradients.asp">Read about gradients</a>]'
      ),
      '#default_value' => isset($settings['css_settings']['image_gradient']) ? $settings['css_settings']['image_gradient'] : NULL,
    ];
    // The media query specifics.
    $element['css_settings']['image_media_query'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Media Query'),
      '#description' => $this->t(
        'Apply this background image css using a media query. CSS3 Only. Useful for responsive designs.
                      Example: only screen and (min-width:481px) and (max-width:768px)
                      [<a href="http://www.w3.org/TR/css3-mediaqueries/">Read about media queries</a>]'
      ),
      '#default_value' => isset($settings['css_settings']['image_media_query']) ? $settings['css_settings']['image_media_query'] : NULL,
    ];
    $element['css_settings']['image_important'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add "!important" to the background property.'),
      '#description' => $this->t(
        'This can be helpful to override any existing background image or color properties added by the theme.'
      ),
      '#default_value' => isset($settings['css_settings']['image_important']) ? $settings['css_settings']['image_important'] : NULL,
    ];
    $element['css_settings']['image_path_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Image Path Format'),
      '#options' => [
        'absolute' => $this->t('Absolute'),
        'relative' => $this->t('Relative'),
      ],
      '#description' => $this->t(
        'Defaults to absolute URLs, however relative URLs maybe solve issues with mixed content errors on websites being served on HTTPS.'
      ),
      '#default_value' => isset($settings['css_settings']['image_path_format']) ? $settings['css_settings']['image_path_format'] : NULL,
    ];

    return $element;
  }


  /**
   * Function taken from the module 'image'.
   *
   * Adds a background image to the page using the
   * css 'background' property.
   *
   * @param string $image_path
   *   The path of the image to use. This can be either
   *      - A relative path e.g. sites/default/files/image.png
   *      - A uri: e.g. public://image.png.
   * @param array $css_settings
   *   An array of css settings to use. Possible values are:
   *      - image_selector: The css selector to use
   *      - image_color: The background color
   *      - image_x: The x offset
   *      - image_y: The y offset
   *      - image_attachment: The attachment property (scroll or fixed)
   *      - image_repeat: The repeat settings
   *      - image_background_size: The background size property if necessary
   *    Default settings will be used for any values not provided.
   * @param string $image_style
   *   Optionally add an image style to the image before applying it to the
   *   background.
   *
   * @return array
   *   The array containing the CSS.
   */
  public function getBackgroundImageCss($image_path, array $css_settings = [], $image_style = NULL) {
    $defaults = self::defaultSettings();
    $css_settings += $defaults['css_settings'];

    // Pull the default css setting if not provided.
    $selector = Xss::filter($css_settings['image_selector']);
    $color = Xss::filter($css_settings['image_color']);
    $x = Xss::filter($css_settings['image_x']);
    $y = Xss::filter($css_settings['image_y']);
    $attachment = $css_settings['image_attachment'];
    $repeat = $css_settings['image_repeat'];
    $important = $css_settings['image_important'];
    $background_size = Xss::filter($css_settings['image_background_size']);
    $background_size_ie8 = $css_settings['image_background_size_ie8'];
    $background_gradient = !empty($css_settings['image_gradient']) ? $css_settings['image_gradient'] . ',' : '';
    $media_query = Xss::filter($css_settings['image_media_query']);
    $z_index = Xss::filter($css_settings['image_z_index']);

    // If important is true, we turn it into a string for css output.
    if ($important) {
      $important = '!important';
    }
    else {
      $important = '';
    }

    // Handle the background size property.
    $size = '';
    $ie_size = '';
    if ($background_size) {
      // CSS3.
      $size = sprintf('background-size: %s %s;', $background_size, $important);
      // Let's cover ourselves for other browsers as well...
      $size .= sprintf('-webkit-background-size: %s %s;', $background_size, $important);
      $size .= sprintf('-moz-background-size: %s %s;', $background_size, $important);
      $size .= sprintf('-o-background-size: %s %s;', $background_size, $important);
      // IE filters to apply the cover effect.
      if ($background_size === 'cover' && $background_size_ie8 && $image_path !== NULL) {
        $ie_size = sprintf(
          "filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='%s', sizingMethod='scale');",
          $image_path
        );
        $ie_size .= sprintf(
          "-ms-filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='%s', sizingMethod='scale');",
          $image_path
        );
      }
    }

    // Add the css if we have everything we need.
    if ($selector) {
      $style = sprintf('%s {', $selector);
      if ($color) {
        $style .= sprintf('background-color: %s %s;', $color, $important);
      }
      if ($image_path !== NULL) {
        $style .= sprintf("background-image: %s url('%s') %s;", $background_gradient, $image_path, $important);
        if ($repeat) {
          $style .= sprintf('background-repeat: %s %s;', $repeat, $important);
        }
        if ($x && $y) {
          $style .= sprintf('background-position: %s %s %s;', $x, $y, $important);
        }
        if ($attachment) {
          $style .= sprintf('background-attachment: %s %s;', $attachment, $important);
        }
        if ($z_index) {
          $style .= sprintf('z-index: %s;', $z_index);
        }
      }

      $style .= $size;
      $style .= $background_size_ie8 ? $ie_size : '';
      $style .= '}';

      return [
        'type' => 'inline',
        'style' => $style,
        'media' => !empty($media_query) ? $media_query : 'all',
        'group' => CSS_THEME,
      ];
    }

    return [];
  }

}
