<?php

namespace Drupal\layout_settings\Plugin\LayoutSettings;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\layout_settings\Plugin\LayoutSettingsBase;
use Drupal\layout_settings\Plugin\LayoutSettingsManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * @LayoutSettings(
 *  id = "custom_classes",
 *  label = @Translation("Custom Classes"),
 *  context = {
 *    "layout",
 *    "rows",
 *    "regions"
 *  }
 * )
 */
class CustomClasses extends LayoutSettingsBase {

  use StringTranslationTrait;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LayoutSettingsManager $layout_settings_manager, Request $request, RendererInterface $renderer) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $layout_settings_manager);
    $this->request = $request;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.layout_settings'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(&$build, $context) {
    $region = $context[1];
    switch ($context[0]) {
      case 'layout':
        if($region === 'wrapper'){
          $build['#parent_attributes']['class'][] = $this->configuration['classes'];
        } else {
          $build['#attributes']['class'][] = $this->configuration['classes'];
        }
        break;

      case 'rows':
        break;

      case 'regions':
        $build[$region]['#attributes']['class'][] = $this->configuration['classes'];
        break;

      default:
        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultValues() {
    return self::defaultSettings() + ['classes' => ''];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array &$form, FormStateInterface $form_state) {
    $element = [];
    $data = $this->configuration;

    $element['classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Classes'),
      '#default_value' => !empty($data['classes']) ? $data['classes'] : NULL,
    ];

    return $element;
  }

}
