<?php

namespace Drupal\layout_settings\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for Layout settings plugins.
 */
abstract class LayoutSettingsBase extends PluginBase implements LayoutSettingsInterface {

  use StringTranslationTrait;

  /**
   * The layout settings manager.
   *
   * @var \Drupal\layout_settings\Plugin\LayoutSettingsManager
   */
  public $layoutSettingsManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LayoutSettingsManager $layout_settings_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->layoutSettingsManager = $layout_settings_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.layout_settings')
    );
  }

  public function get($property) {
    return $this->{$property} ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultValues() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array &$form, FormStateInterface $form_state) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    return;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    return;
  }

  /**
   * {@inheritdoc}
   */
  public function preprocess(array $settings, array &$variables, array $context) {
    return;
  }

  /**
   * {@inheritdoc}
   */
  public function build(&$build, $context){
    return;
  }

}
