<?php

namespace Drupal\layout_settings\Plugin;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;

/**
 * Defines an interface for Layout settings plugins.
 */
interface LayoutSettingsInterface extends PluginInspectionInterface, ContainerFactoryPluginInterface {

  /**
   * Default values of plugin properties.
   *
   * @return array
   *   Array of properties with their default values.
   */
  public function defaultValues();

  /**
   * Form array of this plugin.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form_state of layout plugin form.
   *
   * @return array
   *   The result array form for this plugin.
   */
  public function buildForm(array &$form, FormStateInterface $form_state);

  /**
   * Callback method for validate the form submision.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form_state of layout plugin form.
   */
  public function validateForm(array &$form, FormStateInterface $form_state);

  /**
   * Callback method to do when form is submited.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form_state of layout plugin form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state);

  /**
   * Layout template preprocess and is used by plugin for apply their settings.
   *
   * @param array $settings
   *   The instance plugin settings.
   * @param array $variables
   *   The preprocess layout variables.
   * @param array $context
   *   The context where to apply the settings.
   */
  public function preprocess(array $settings, array &$variables, array $context);

  /**
   * Callback method to apply the settings when layout is buildind.
   *
   * @param $build
   *   The render array of build.
   * @param $context
   *  The context where is apply the settings.
   * @return mixed
   */
  public function build(&$build, $context);

}
